@@ -29,6 +29,9 @@
 #include "browser.h"
 #include "message.h"
 #include "imap_private.h"
+#ifdef USE_SSL
+# include "mutt_ssl.h"
+#endif
 
 #include <unistd.h>
 #include <ctype.h>
@@ -45,6 +48,50 @@ static int imap_check_capabilities (IMAP_DATA* idata);
 static void imap_set_flag (IMAP_DATA* idata, int aclbit, int flag,
 			   const char* str, char* flags, size_t flsize);
 
+/* imap_access: Check permissions on an IMAP mailbox. */
+int imap_access (const char* path, int flags)
+{
+  IMAP_DATA* idata;
+  IMAP_MBOX mx;
+  char buf[LONG_STRING];
+  char mailbox[LONG_STRING];
+  char mbox[LONG_STRING];
+
+  if (imap_parse_path (path, &mx))
+    return -1;
+
+  if (!(idata = imap_conn_find (&mx.account,
+    option (OPTIMAPPASSIVE) ? M_IMAP_CONN_NONEW : 0)))
+  {
+    FREE (&mx.mbox);
+    return -1;
+  }
+  
+
+  imap_fix_path (idata, mx.mbox, mailbox, sizeof (mailbox));
+  FREE (&mx.mbox);
+  imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
+
+  /* TODO: ACL checks. Right now we assume if it exists we can mess with it. */
+  if (mutt_bit_isset (idata->capabilities, IMAP4REV1))
+    snprintf (buf, sizeof (buf), "STATUS %s (UIDVALIDITY)", mbox);
+  else if (mutt_bit_isset (idata->capabilities, STATUS))
+    snprintf (buf, sizeof (buf), "STATUS %s (UID-VALIDITY)", mbox);
+  else
+  {
+    dprint (2, (debugfile, "imap_access: STATUS not supported?\n"));
+    return -1;
+  }
+
+  if (imap_exec (idata, buf, IMAP_CMD_FAIL_OK) < 0)
+  {
+    dprint (1, (debugfile, "imap_access: Can't check STATUS of %s\n", mbox));
+    return -1;
+  }
+
+  return 0;
+}
+
 int imap_create_mailbox (IMAP_DATA* idata, char* mailbox)
 {
   char buf[LONG_STRING], mbox[LONG_STRING];
@@ -309,6 +356,7 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
 int imap_open_connection (IMAP_DATA* idata)
 {
   char buf[LONG_STRING];
+  int rc;
 
   if (mutt_socket_open (idata->conn) < 0)
   {
@@ -324,8 +372,42 @@ int imap_open_connection (IMAP_DATA* idata)
 
   if (mutt_strncmp ("* OK", idata->cmd.buf, 4) == 0)
   {
-    if (imap_check_capabilities (idata) || imap_authenticate (idata))
+    /* TODO: Parse new tagged CAPABILITY data (* OK [CAPABILITY...]) */
+    if (imap_check_capabilities (idata))
       goto bail;
+#if defined(USE_SSL) && !defined(USE_NSS)
+    /* Attempt STARTTLS if available and desired. */
+    if (mutt_bit_isset (idata->capabilities, STARTTLS))
+    {
+      if ((rc = query_quadoption (OPT_SSLSTARTTLS,
+        _("Secure connection with TLS?"))) == -1)
+	goto bail;
+      if (rc == M_YES) {
+	if ((rc = imap_exec (idata, "STARTTLS", IMAP_CMD_FAIL_OK)) == -1)
+	  goto bail;
+	if (rc != -2)
+	{
+	  if (mutt_ssl_starttls (idata->conn))
+	  {
+	    mutt_error ("Could not negotiate TLS connection");
+	    sleep (1);
+	    goto bail;
+	  }
+	  else
+	  {
+	    /* RFC 2595 demands we recheck CAPABILITY after TLS completes. */
+	    if (imap_exec (idata, "CAPABILITY", 0))
+	      goto bail;
+	  }
+	}
+      }
+    }
+#endif    
+    if (imap_authenticate (idata))
+      goto bail;
+    if (idata->conn->ssf)
+      dprint (2, (debugfile, "Communication encrypted at %d bits\n",
+	idata->conn->ssf));
   }
   else if (mutt_strncmp ("* PREAUTH", idata->cmd.buf, 9) == 0)
   {
@@ -971,7 +1053,8 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
   idata = (IMAP_DATA*) ctx->data;
 
   now = time(NULL);
-  if (now > ImapLastCheck + Timeout) {
+  if (now > ImapLastCheck + Timeout)
+  {
     ImapLastCheck = now;
 
     if (imap_exec (idata, "NOOP", 0) != 0)
@@ -1130,6 +1213,9 @@ int imap_parse_list_response(IMAP_DATA* idata, char **name, int *noselect,
 	  *noselect = 1;
 	if (!strncasecmp (s, "\\NoInferiors", 12))
 	  *noinferiors = 1;
+	/* See draft-gahrns-imap-child-mailbox-?? */
+	if (!strncasecmp (s, "\\HasNoChildren", 14))
+	  *noinferiors = 1;
 	if (*s != ')')
 	  s++;
 	while (*s && *s != '\\' && *s != ')') s++;
