@@ -262,7 +262,7 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
 	tok->dsize = expnlen + mutt_strlen (tok->dptr) + 1;
 	ptr = safe_malloc (tok->dsize);
 	memcpy (ptr, expn.data, expnlen);
-	strcpy (ptr + expnlen, tok->dptr);
+	strcpy (ptr + expnlen, tok->dptr);	/* __STRCPY_CHECKED__ */
 	if (tok->destroy)
 	  FREE (&tok->data);
 	tok->data = ptr;
@@ -1758,7 +1758,7 @@ void mutt_init (int skip_sys_rc, LIST *commands)
     if (*DOMAIN != '@')
   {
     Fqdn = safe_malloc (mutt_strlen (DOMAIN) + mutt_strlen (Hostname) + 2);
-    sprintf (Fqdn, "%s.%s", NONULL(Hostname), DOMAIN);
+    sprintf (Fqdn, "%s.%s", NONULL(Hostname), DOMAIN);	/* __SPRINTF_CHECKED__ */
   }
   else
     Fqdn = safe_strdup(NONULL(Hostname));
