@@ -146,12 +146,44 @@ static void pgp_current_time (STATE *s)
 }
 
 
+/* copy PGP output messages and look for signs of a good signature */
+
+static int pgp_copy_checksig (FILE *fpin, FILE *fpout)
+{
+  int rv = -1;
+
+  if (PgpGoodSign.pattern)
+  {
+    char *line = NULL;
+    int lineno = 0;
+    size_t linelen;
+    
+    while ((line = mutt_read_line (line, &linelen, fpin, &lineno)) != NULL)
+    {
+      if (regexec (PgpGoodSign.rx, line, 0, NULL, 0) == 0)
+	rv = 0;
+      
+      fputs (line, fpout);
+      fputc ('\n', fpout);
+    }
+    safe_free ((void **) &line);
+  }
+  else
+  {
+    mutt_copy_stream (fpin, fpout);
+    rv = 1;
+  }
+
+  return rv;
+}
+
+
 /* Support for the Application/PGP Content Type. */
 
 void pgp_application_pgp_handler (BODY *m, STATE *s)
 {
   int needpass = -1, pgp_keyblock = 0;
-  int clearsign = 0;
+  int clearsign = 0, rv;
   long start_pos = 0;
   long bytes, last_pos, offset;
   char buf[HUGE_STRING];
@@ -161,6 +193,9 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
   FILE *tmpfp;
   pid_t thepid;
 
+  short maybe_goodsig = 1;
+  short have_any_sigs = 0;
+  
   fseek (s->fpin, m->offset, 0);
   last_pos = m->offset;
   
@@ -198,6 +233,8 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
 	continue;
       }
 
+      have_any_sigs = have_any_sigs || clearsign;
+      
       if(!clearsign || s->flags & M_VERIFY)
       {
 
@@ -256,18 +293,20 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
 	  fputc ('\n', pgpin);
 	}
 
-	fclose (pgpin);
+	safe_fclose (&pgpin);
 	
 	if (s->flags & M_DISPLAY)
 	  pgp_current_time (s);
 	
-	mutt_wait_filter (thepid);
+	rv = mutt_wait_filter (thepid);
 
 	mutt_unlink(tmpfname);
 	
 	if (s->flags & M_DISPLAY)
-	  mutt_copy_stream(pgperr, s->fpout);
-	fclose (pgperr);
+	  if (pgp_copy_checksig (pgperr, s->fpout) != 0 || rv != 0)
+	    maybe_goodsig = 0;
+	
+	safe_fclose (&pgperr);
 	
 	if (s->flags & M_DISPLAY)
 	  state_puts (_("\n[-- End of PGP output --]\n\n"), s);
@@ -304,9 +343,8 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
       
       if(pgpout)
       {
-	fclose (pgpout);
-	pgpout = NULL;
-	mutt_unlink(outfile);
+	safe_fclose (&pgpout);
+	mutt_unlink (outfile);
       }
 
       /* decode clearsign stuff */
@@ -397,12 +435,83 @@ void pgp_application_pgp_handler (BODY *m, STATE *s)
     }
   }
 
+  m->goodsig = (maybe_goodsig && have_any_sigs);
+
   if (needpass == -1)
   {
     state_puts (_("[-- Error: could not find beginning of PGP message! --]\n\n"), s);
     return;
   }
+}
 
+static int pgp_check_traditional_one_body (FILE *fp, BODY *b, int tagged_only)
+{
+  char tempfile[_POSIX_PATH_MAX];
+  char buf[HUGE_STRING];
+  FILE *tfp;
+  
+  short sgn = 0;
+  short enc = 0;
+  
+  if (b->type != TYPETEXT)
+    return 0;
+
+  if (tagged_only && !b->tagged)
+    return 0;
+  
+  mutt_mktemp (tempfile);
+  if (mutt_decode_save_attachment (fp, b, tempfile, 0, 0) != 0)
+  {
+    unlink (tempfile);
+    return 0;
+  }
+  
+  if ((tfp = fopen (tempfile, "r")) == NULL)
+  {
+    unlink (tempfile);
+    return 0;
+  }
+    
+  while (fgets (buf, sizeof (buf), tfp))
+  {
+    if (mutt_strncmp ("-----BEGIN PGP ", buf, 15) == 0)
+    {
+      if (mutt_strcmp ("MESSAGE-----\n", buf + 15) == 0)
+	enc = 1;
+      else if (mutt_strcmp ("SIGNED MESSAGE-----\n", buf + 15) == 0)
+	sgn = 1;
+    }
+  }
+  
+  safe_fclose (&tfp);
+  unlink (tempfile);
+
+  if (!enc && !sgn)
+    return 0;
+
+  /* fix the content type */
+  
+  b->type = TYPEAPPLICATION;
+  mutt_str_replace (&b->subtype, "pgp");
+  
+  mutt_set_parameter ("format", "text", &b->parameter);
+  mutt_set_parameter ("x-action", enc ? "encrypt" : "sign", &b->parameter);
+  
+  return 1;
+}
+
+int pgp_check_traditional (FILE *fp, BODY *b, int tagged_only)
+{
+  int rv = 0;
+  for (; b; b = b->next)
+  {
+    if (is_multipart (b))
+      rv = pgp_check_traditional (fp, b->parts, tagged_only) || rv;
+    else if (b->type == TYPETEXT)
+      rv = pgp_check_traditional_one_body (fp, b, tagged_only) || rv;
+  }
+
+  return rv;
 }
 
 int mutt_is_multipart_signed (BODY *b)
@@ -466,21 +575,22 @@ int pgp_query (BODY *m)
 {
   int t = 0;
 
-  t |= mutt_is_application_pgp(m);
-  
-  /* Check for PGP/MIME messages. */
-  if (m->type == TYPEMULTIPART)
+  t |= mutt_is_application_pgp (m);
+
+  if ((t & PGPSIGN) && m->goodsig)
+    t |= PGPGOODSIGN;
+  else if (m->type == TYPEMULTIPART)
   {
     if (mutt_is_multipart_signed(m))
       t |= PGPSIGN;
     else if (mutt_is_multipart_encrypted(m))
       t |= PGPENCRYPT;
 
-    if ((mutt_is_multipart_signed (m) || mutt_is_multipart_encrypted (m)) 
-	&& m->goodsig)
+    if (m->goodsig && 
+	(mutt_is_multipart_signed (m) || mutt_is_multipart_encrypted (m)))
       t |= PGPGOODSIGN;
   }
-
+  
   if (m->type == TYPEMULTIPART || m->type == TYPEMESSAGE)
   {
     BODY *p;
@@ -549,6 +659,7 @@ static int pgp_write_signed(BODY *a, STATE *s, const char *tempfile)
   return 0;
 }
 
+
 static int pgp_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
 {
   char sigfile[_POSIX_PATH_MAX], pgperrfile[_POSIX_PATH_MAX];
@@ -582,33 +693,15 @@ static int pgp_verify_one (BODY *sigbdy, STATE *s, const char *tempfile)
 				   -1, -1, fileno(pgperr),
 				   tempfile, sigfile)) != -1)
   {
-    if (PgpGoodSign.pattern)
-    {
-      char *line = NULL;
-      int lineno = 0;
-      size_t linelen;
-
-      while ((line = mutt_read_line (line, &linelen, pgpout, &lineno)) != NULL)
-      {
-	if (regexec (PgpGoodSign.rx, line, 0, NULL, 0) == 0)
-	  badsig = 0;
-
-	fputs (line, s->fpout);
-	fputc ('\n', s->fpout);
-      }
-      safe_free ((void **) &line);
-    }
-    else
-    {
-      mutt_copy_stream(pgpout, s->fpout);
+    if (pgp_copy_checksig (pgpout, s->fpout) >= 0)
       badsig = 0;
-    }
-
-    fclose (pgpout);
-    fflush(pgperr);
-    rewind(pgperr);
-    mutt_copy_stream(pgperr, s->fpout);
-    fclose(pgperr);
+    
+    
+    safe_fclose (&pgpout);
+    fflush (pgperr);
+    rewind (pgperr);
+    mutt_copy_stream (pgperr, s->fpout);
+    safe_fclose (&pgperr);
     
     if (mutt_wait_filter (thepid))
       badsig = -1;
